@@ -19,17 +19,61 @@ DEFAULT_NODATA <- list("Byte" = 255, "Int8" = -128,
 					   "Float32" = -99999.0, "Float64" = -99999.0)
 
 
+#' List of default DEM processing options
+#'
+#' These values are used in `dem_proc()` as the default processing options:
+#' \preformatted{
+#'     list(hillshade = c("-z", "1", "-s", "1", "-az", "315",
+#'                        "-alt", "45", "-alg", "Horn",
+#'                        "-combined", "-compute_edges"),
+#'          slope = c("-s", "1", "-alg", "Horn", "-compute_edges"),
+#'          aspect = c("-alg", "Horn", "-compute_edges"),
+#'          color_relief = character(),
+#'          TRI = c("-alg", "Riley", "-compute_edges"),
+#'          TPI = c("-compute_edges"),
+#'          roughness = c("-compute_edges"))
+#' }
+#' @seealso
+#' [dem_proc()]
+#'
+#' \url{https://gdal.org/programs/gdaldem.html} for a description of all
+#' available command-line options for each processing mode
+#' @export
+DEFAULT_DEM_PROC <- list(hillshade = c("-z", "1", "-s", "1", "-az", "315",
+									   "-alt", "45", "-alg", "Horn",
+									   "-combined", "-compute_edges"),
+						slope = c("-s", "1", "-alg", "Horn", "-compute_edges"),
+						aspect = c("-alg", "Horn", "-compute_edges"),
+						color_relief = character(),
+						TRI = c("-alg", "Riley", "-compute_edges"),
+						TPI = c("-compute_edges"),
+						roughness = c("-compute_edges"))
+
+
+#' @noRd
+.VRT_KERNEL_TEMPLATE <- 
+"<KernelFilteredSource>
+  <SourceFilename relativeToVRT=\"%d\">%s</SourceFilename><SourceBand>%d</SourceBand>
+  <SrcRect xOff=\"%d\" yOff=\"%d\" xSize=\"%d\" ySize=\"%d\"/>
+  <DstRect xOff=\"0\" yOff=\"0\" xSize=\"%d\" ySize=\"%d\"/>
+  <Kernel normalized=\"%d\">
+    <Size>%d</Size>
+    <Coefs>%s</Coefs>
+  </Kernel>
+</KernelFilteredSource>"
+
+
 #' @noRd
 .getGDALformat <- function(file) {
 # Only for guessing common output formats
 	file <- as.character(file)
-	if (endsWith(file, ".img")) {
+	if (endsWith(file, ".img") || endsWith(file, ".IMG")) {
 		return("HFA")
 	}
-	if (endsWith(file, ".tif")) {
+	if (endsWith(file, ".tif") || endsWith(file, ".TIF")) {
 		return("GTiff")
 	}
-	if (endsWith(file, ".vrt")) {
+	if (endsWith(file, ".vrt") || endsWith(file, ".VRT")) {
 		return("VRT")
 	}
 	return(NULL)
@@ -47,19 +91,6 @@ DEFAULT_NODATA <- list("Byte" = 255, "Int8" = -128,
 }
 
 
-#' @noRd
-.VRT_KERNEL_TEMPLATE <- 
-"<KernelFilteredSource>
-  <SourceFilename relativeToVRT=\"%d\">%s</SourceFilename><SourceBand>%d</SourceBand>
-  <SrcRect xOff=\"%d\" yOff=\"%d\" xSize=\"%d\" ySize=\"%d\"/>
-  <DstRect xOff=\"0\" yOff=\"0\" xSize=\"%d\" ySize=\"%d\"/>
-  <Kernel normalized=\"%d\">
-    <Size>%d</Size>
-    <Coefs>%s</Coefs>
-  </Kernel>
-</KernelFilteredSource>"
-
-
 #' Convenience wrapper for `GDALRaster$read()`
 #' 
 #' @description
@@ -640,7 +671,7 @@ rasterToVRT <- function(srcfile, relativeToVRT = FALSE,
 				sqrt(length(krnl)),
 				paste(krnl, collapse=" ")) )
 				
-			vrt_ds$.setMetadataItem(band, "source_0", krnl_xml, "vrt_sources")
+			vrt_ds$setMetadataItem(band, "source_0", krnl_xml, "vrt_sources")
 		}
 		vrt_ds$close()
 	}
@@ -936,7 +967,7 @@ calc <- function(expr,
 		for(r in rasterfiles) {
 			ds <- new(GDALRaster, r, TRUE)
 			if(ds$getRasterYSize() != nrows || ds$getRasterXSize() != ncols) {
-				message(rasterfiles[r])
+				message(r)
 				stop("All input rasters must have the same dimensions.", 
 					call. = FALSE)
 			}
@@ -1165,3 +1196,46 @@ combine <- function(rasterfiles, var.names=NULL, bands=NULL,
 	df <- .combine(rasterfiles, var.names, bands, dstfile, fmt, dtName, options)
 	return(df)
 }
+
+
+#' GDAL DEM processing
+#' 
+#' @description
+#' `dem_proc()` generates DEM derivatives from an input elevation raster. This
+#' function is a wrapper for the \command{gdaldem} command-line utility.
+#' See \url{https://gdal.org/programs/gdaldem.html} for details.
+#'
+#' @param mode Character. Name of the DEM processing mode. One of hillshade,
+#' slope, aspect, color-relief, TRI, TPI or roughness.
+#' @param srcfile Filename of the the source elevation raster.
+#' @param dstfile Filename of the output raster.
+#' @param mode_options An optional character vector of command-line options
+#' (see [DEFAULT_DEM_PROC] for default values).
+#' @param color_file Filename of a text file containing lines formatted as:
+#' "elevation_value red green blue". Only used when `mode = "color-relief"`.
+#' @returns Logical indicating success (invisible `TRUE`).
+#' An error is raised if the operation fails.
+#'
+#' @note
+#' Band 1 of the source elevation raster is read by default, but this can be
+#' changed by including a `-b` command-line argument in `mode_options`.
+#' See the \href{https://gdal.org/programs/gdaldem.html}{documentation for
+#' `gdaldem`} for a description of all available options for each processing
+#' mode.
+#'
+#' @examples
+#' elev_file <- system.file("extdata/storml_elev.tif", package="gdalraster")
+#' slp_file <- paste0(tempdir(), "/", "storml_slp.tif")
+#' dem_proc("slope", elev_file, slp_file)
+#' @export
+dem_proc <- function(mode,
+					srcfile,
+					dstfile, 
+					mode_options=DEFAULT_DEM_PROC[[mode]],
+					color_file=NULL) {
+
+	if (is.null(DEFAULT_DEM_PROC[[mode]]))
+		stop("DEM processing mode not recognized.", call.=FALSE)
+
+	invisible(.dem_proc(mode, srcfile, dstfile, mode_options, color_file))
+}
